"use strict";
var core_1 = require('@angular/core');
/**
  * @private
  * Select all of the HTML text elements with the color attribute to apply the text-color class.
 */
var Typography = (function () {
    function Typography(_elementRef, _renderer) {
        this._elementRef = _elementRef;
        this._renderer = _renderer;
    }
    Object.defineProperty(Typography.prototype, "color", {
        /**
         * @input {string} The predefined color to use. For example: `"primary"`, `"secondary"`, `"danger"`.
         */
        get: function () {
            return this._color;
        },
        set: function (val) {
            this._updateColor(val);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @private
     */
    Typography.prototype._updateColor = function (newColor) {
        this._setElementColor(this._color, false);
        this._setElementColor(newColor, true);
        this._color = newColor;
    };
    /**
     * @private
     */
    Typography.prototype._setElementColor = function (color, isAdd) {
        if (color !== null && color !== '') {
            this._renderer.setElementClass(this._elementRef.nativeElement, "text-" + color, isAdd);
        }
    };
    Typography.decorators = [
        { type: core_1.Directive, args: [{
                    selector: 'h1[color], h2[color], h3[color], h4[color], h5[color], h6[color], a[color], p[color], span[color], b[color], i[color], strong[color], em[color], small[color], sub[color], sup[color]'
                },] },
    ];
    /** @nocollapse */
    Typography.ctorParameters = [
        { type: core_1.ElementRef, },
        { type: core_1.Renderer, },
    ];
    Typography.propDecorators = {
        'color': [{ type: core_1.Input },],
    };
    return Typography;
}());
exports.Typography = Typography;
