import { OpaqueToken } from '@angular/core';
import { Location } from '@angular/common';
import { App } from '../components/app/app';
import { NavSegment } from './nav-util';
import { NavController } from './nav-controller';
import { Tab } from '../components/tabs/tab';
import { Tabs } from '../components/tabs/tabs';
import { UrlSerializer } from './url-serializer';
import { ViewController } from './view-controller';
/**
 * Deep Linking Scenarios:
 * 1) Initialize all NavControllers using the initial browser URL
 * 2) User clicks browser back button
 * 3) User clicks browser forward button
 * 4) User changes browser URL
 * 5) User clicks link href
 * 6) App uses NavController push/pop/setRoot/insert/remove
 *
 * Terms:
 * - URL: The string value found in the browser's URL bar
 * - Segment: Deep linker's data about each section between / in the URL
 * - Path: Deep linker's array of segments
 * - History: Deep linker's string array of internal URL history
 * - Location: Angular's Location provider, which abstracts Hash/Path Location Strategies
 */
export declare class DeepLinker {
    app: App;
    serializer: UrlSerializer;
    location: Location;
    segments: NavSegment[];
    history: string[];
    indexAliasUrl: string;
    constructor(app: App, serializer: UrlSerializer, location: Location);
    init(): void;
    /**
     * The browser's location has been updated somehow.
     */
    urlChange(browserUrl: string): void;
    /**
     * Update the deep linker using the NavController's current active view.
     */
    navChange(direction: string): void;
    updateLocation(browserUrl: string, direction: string): void;
    getComponentFromName(componentName: any): any;
    createUrl(nav: any, nameOrComponent: any, data: any, prepareExternalUrl?: boolean): string;
    /**
     * Build a browser URL out of this NavController. Climbs up the tree
     * of NavController's to create a string representation of all the
     * NavControllers state.
     */
    pathFromNavs(nav: NavController, component?: any, data?: any): NavSegment[];
    getTabSelector(tab: Tab): string;
    getSelectedTabIndex(tabsNav: Tabs, pathName: string, fallbackIndex?: number): number;
    /**
     * Each NavController will call this method when it initializes for
     * the first time. This allows each NavController to figure out
     * where it lives in the path and load up the correct component.
     */
    initNav(nav: any): NavSegment;
    initViews(segment: NavSegment): ViewController[];
    /**
     * Using the known Path of Segments, walk down all descendents
     * from the root NavController and load each NavController according
     * to each Segment. This is usually called after a browser URL and
     * Path changes and needs to update all NavControllers to match
     * the new browser URL. Because the URL is already known, it will
     * not update the browser's URL when transitions have completed.
     */
    loadNavFromPath(nav: NavController, done?: Function): void;
    loadViewFromSegment(navInstance: any, done: Function): void;
    isBackUrl(browserUrl: string): boolean;
    isCurrentUrl(browserUrl: string): boolean;
    historyPush(browserUrl: string): void;
    historyPop(): void;
}
export declare function setupDeepLinker(app: App, serializer: UrlSerializer, location: Location): DeepLinker;
export declare const UserDeepLinkConfig: OpaqueToken;
export declare function provideDeepLinker(userDeepLinkConfig: any): any[];
export declare function normalizeUrl(browserUrl: string): string;
